//
// © 2005-2008 Marko Lindqvist
//

#ifndef H_SDKS_SUDOKU
#define H_SDKS_SUDOKU

// Untested solution method.
// #define ENABLE_REVEAL_PAIRING

enum construct_type { CT_ROW, CT_COL, CT_REG };

// Order of methods should be from simple to more complex, since
// first ones are attempted first and only when they fail are later ones used
enum solving_method { METHOD_BASE = 0,
                      METHOD_ROWCOL_FOR_REGION,
                      METHOD_REGION_FOR_ROWCOL,
                      METHOD_PAIRING,
                      METHOD_REVEAL_PAIRING,
                      METHOD_LAST };

class cell;
class sdks_ui;

class sudoku
{
 public:
  sudoku();
  sudoku(int size);
  sudoku(int size, sdks_ui *ui_in);
  ~sudoku();
  bool read();
  bool solve();
  bool check_legality();
  int get_size();
  sdks_ui *get_ui();
  cell *get_cell(construct_type construct, int cnum, int index);
 private:
  bool apply_solving_method(solving_method method);
  bool solve_base(construct_type construct, int cnum);
  bool solve_slow_rowcol(construct_type construct, int row);
  bool solve_region_slow(int region);
  bool solve_pairing(construct_type construct, int cnum);
  bool solve_reveal_pairing(construct_type construct, int cnum);
  int get_reg(construct_type construct, int cnum, int index);
  cell *get_row_cell(int row, int index);
  cell *get_col_cell(int col, int index);
  cell *get_reg_cell(int reg, int index);
  cell *get_regxy_cell(int reg, int x, int y);
  int region_origo_x(int region);
  int region_origo_y(int region);
  int linear_to_x(int region, int linear);
  int linear_to_y(int region, int linear);
  bool region_x(int region, int x);
  bool region_y(int region, int y);
  const char *construct_name(construct_type construct);
  cell ***cells;
  int unsolved;
  int base_size;
  int dimension;
  sdks_ui *ui;
  bool ui_alloc;    // UI allocated for this sudoku only?
};


#endif // H_SDKS_SUDOKU
